@@ -802,13 +802,13 @@ impl JsRuntime {
     let module_map = module_map_rc.borrow();
 
     let has_pending_ops = !state.pending_ops.is_empty();
-
     let has_pending_dyn_imports = module_map.has_pending_dynamic_imports();
     let has_pending_dyn_module_evaluation =
       !state.pending_dyn_mod_evaluate.is_empty();
     let has_pending_module_evaluation = state.pending_mod_evaluate.is_some();
     let has_pending_background_tasks =
       self.v8_isolate().has_pending_background_tasks();
+    let has_tick_scheduled = state.has_tick_scheduled;
     let inspector_has_active_sessions = self
       .inspector
       .as_ref()
@@ -820,6 +820,7 @@ impl JsRuntime {
       && !has_pending_dyn_module_evaluation
       && !has_pending_module_evaluation
       && !has_pending_background_tasks
+      && !has_tick_scheduled
     {
       if wait_for_inspector && inspector_has_active_sessions {
         return Poll::Pending;
@@ -1017,8 +1018,9 @@ impl JsRuntime {
     // https://github.com/denoland/deno/issues/4908
     // https://v8.dev/features/top-level-await#module-execution-order
     let scope = &mut self.handle_scope();
-    let module = v8::Local::new(scope, &module_handle);
-    let maybe_value = module.evaluate(scope);
+    let tc_scope = &mut v8::TryCatch::new(scope);
+    let module = v8::Local::new(tc_scope, &module_handle);
+    let maybe_value = module.evaluate(tc_scope);
 
     // Update status after evaluating.
     let status = module.get_status();
@@ -1033,12 +1035,12 @@ impl JsRuntime {
       let empty_fn = |_scope: &mut v8::HandleScope,
                       _args: v8::FunctionCallbackArguments,
                       _rv: v8::ReturnValue| {};
-      let empty_fn = v8::FunctionTemplate::new(scope, empty_fn);
-      let empty_fn = empty_fn.get_function(scope).unwrap();
-      promise.catch(scope, empty_fn);
+      let empty_fn = v8::FunctionTemplate::new(tc_scope, empty_fn);
+      let empty_fn = empty_fn.get_function(tc_scope).unwrap();
+      promise.catch(tc_scope, empty_fn);
       let mut state = state_rc.borrow_mut();
-      let promise_global = v8::Global::new(scope, promise);
-      let module_global = v8::Global::new(scope, module);
+      let promise_global = v8::Global::new(tc_scope, promise);
+      let module_global = v8::Global::new(tc_scope, module);
 
       let dyn_import_mod_evaluate = DynImportModEvaluate {
         load_id,
@@ -1048,6 +1050,10 @@ impl JsRuntime {
       };
 
       state.pending_dyn_mod_evaluate.push(dyn_import_mod_evaluate);
+    } else if tc_scope.has_terminated() || tc_scope.is_execution_terminating() {
+      return Err(
+        generic_error("Cannot evaluate dynamically imported module, because JavaScript execution has been terminated.")
+      );
     } else {
       assert!(status == v8::ModuleStatus::Errored);
     }
@@ -1074,11 +1080,12 @@ impl JsRuntime {
     let state_rc = Self::state(self.v8_isolate());
     let module_map_rc = Self::module_map(self.v8_isolate());
     let scope = &mut self.handle_scope();
+    let tc_scope = &mut v8::TryCatch::new(scope);
 
     let module = module_map_rc
       .borrow()
       .get_handle(id)
-      .map(|handle| v8::Local::new(scope, handle))
+      .map(|handle| v8::Local::new(tc_scope, handle))
       .expect("ModuleInfo not found");
     let mut status = module.get_status();
     assert_eq!(status, v8::ModuleStatus::Instantiated);
@@ -1101,7 +1108,7 @@ impl JsRuntime {
     // For more details see:
     // https://github.com/denoland/deno/issues/4908
     // https://v8.dev/features/top-level-await#module-execution-order
-    let maybe_value = module.evaluate(scope);
+    let maybe_value = module.evaluate(tc_scope);
 
     // Update status after evaluating.
     status = module.get_status();
@@ -1113,10 +1120,10 @@ impl JsRuntime {
       );
       let promise = v8::Local::<v8::Promise>::try_from(value)
         .expect("Expected to get promise as module evaluation result");
-      let promise_global = v8::Global::new(scope, promise);
+      let promise_global = v8::Global::new(tc_scope, promise);
       let mut state = state_rc.borrow_mut();
       state.pending_promise_exceptions.remove(&promise_global);
-      let promise_global = v8::Global::new(scope, promise);
+      let promise_global = v8::Global::new(tc_scope, promise);
       assert!(
         state.pending_mod_evaluate.is_none(),
         "There is already pending top level module evaluation"
@@ -1126,7 +1133,11 @@ impl JsRuntime {
         promise: promise_global,
         sender,
       });
-      scope.perform_microtask_checkpoint();
+      tc_scope.perform_microtask_checkpoint();
+    } else if tc_scope.has_terminated() || tc_scope.is_execution_terminating() {
+      sender.send(Err(
+        generic_error("Cannot evaluate module, because JavaScript execution has been terminated.")
+      )).expect("Failed to send module evaluation error.");
     } else {
       assert!(status == v8::ModuleStatus::Errored);
     }
@@ -1632,6 +1643,7 @@ impl JsRuntime {
 pub mod tests {
   use super::*;
   use crate::error::custom_error;
+  use crate::modules::ModuleSource;
   use crate::modules::ModuleSourceFuture;
   use crate::op_async;
   use crate::op_sync;
@@ -2464,4 +2476,124 @@ assertEquals(1, notify_return_value);
     assert_eq!(state.js_macrotask_cbs.len(), 2);
     assert_eq!(state.js_nexttick_cbs.len(), 2);
   }
+
+  #[tokio::test]
+  async fn test_has_tick_scheduled() {
+    run_in_task(|cx| {
+      let macrotask = Arc::new(AtomicUsize::default());
+      let macrotask_ = Arc::clone(&macrotask);
+
+      let next_tick = Arc::new(AtomicUsize::default());
+      let next_tick_ = Arc::clone(&next_tick);
+
+      let op_macrotask = move |_: &mut OpState, _: (), _: ()| {
+        macrotask_.fetch_add(1, Ordering::Relaxed);
+        Ok(())
+      };
+
+      let op_next_tick = move |_: &mut OpState, _: (), _: ()| {
+        next_tick_.fetch_add(1, Ordering::Relaxed);
+        Ok(())
+      };
+
+      let extension = Extension::builder()
+        .ops(vec![("op_macrotask", op_sync(op_macrotask))])
+        .ops(vec![("op_next_tick", op_sync(op_next_tick))])
+        .build();
+
+      let mut runtime = JsRuntime::new(RuntimeOptions {
+        extensions: vec![extension],
+        ..Default::default()
+      });
+
+      runtime
+        .execute_script(
+          "has_tick_scheduled.js",
+          r#"
+          Deno.core.setMacrotaskCallback(() => {
+            Deno.core.opSync("op_macrotask");
+            return true; // We're done.
+          });
+          Deno.core.setNextTickCallback(() => Deno.core.opSync("op_next_tick"));
+          Deno.core.setHasTickScheduled(true);
+          "#,
+        )
+        .unwrap();
+      assert!(matches!(runtime.poll_event_loop(cx, false), Poll::Pending));
+      assert_eq!(1, macrotask.load(Ordering::Relaxed));
+      assert_eq!(1, next_tick.load(Ordering::Relaxed));
+      assert!(matches!(runtime.poll_event_loop(cx, false), Poll::Pending));
+      assert!(matches!(runtime.poll_event_loop(cx, false), Poll::Pending));
+      assert!(matches!(runtime.poll_event_loop(cx, false), Poll::Pending));
+      let state_rc = JsRuntime::state(runtime.v8_isolate());
+      state_rc.borrow_mut().has_tick_scheduled = false;
+      assert!(matches!(
+        runtime.poll_event_loop(cx, false),
+        Poll::Ready(Ok(()))
+      ));
+      assert!(matches!(
+        runtime.poll_event_loop(cx, false),
+        Poll::Ready(Ok(()))
+      ));
+    });
+  }
+
+  #[test]
+  fn terminate_during_module_eval() {
+    #[derive(Default)]
+    struct ModsLoader;
+
+    impl ModuleLoader for ModsLoader {
+      fn resolve(
+        &self,
+        specifier: &str,
+        referrer: &str,
+        _is_main: bool,
+      ) -> Result<ModuleSpecifier, Error> {
+        assert_eq!(specifier, "file:///main.js");
+        assert_eq!(referrer, ".");
+        let s = crate::resolve_import(specifier, referrer).unwrap();
+        Ok(s)
+      }
+
+      fn load(
+        &self,
+        _module_specifier: &ModuleSpecifier,
+        _maybe_referrer: Option<ModuleSpecifier>,
+        _is_dyn_import: bool,
+      ) -> Pin<Box<ModuleSourceFuture>> {
+        async move {
+          Ok(ModuleSource {
+            code: "console.log('hello world');".to_string(),
+            module_url_specified: "file:///main.js".to_string(),
+            module_url_found: "file:///main.js".to_string(),
+          })
+        }
+        .boxed_local()
+      }
+    }
+
+    let loader = std::rc::Rc::new(ModsLoader::default());
+    let mut runtime = JsRuntime::new(RuntimeOptions {
+      module_loader: Some(loader),
+      ..Default::default()
+    });
+
+    let specifier = crate::resolve_url("file:///main.js").unwrap();
+    let source_code = "Deno.core.print('hello\\n')".to_string();
+
+    let module_id = futures::executor::block_on(
+      runtime.load_main_module(&specifier, Some(source_code)),
+    )
+    .unwrap();
+
+    runtime.v8_isolate().terminate_execution();
+
+    let mod_result =
+      futures::executor::block_on(runtime.mod_evaluate(module_id)).unwrap();
+    assert!(mod_result
+      .unwrap_err()
+      .to_string()
+      .contains("JavaScript execution has been terminated"));
+  }
 }
